-- ----------------------------
-- Table structure for ms_report
-- ----------------------------
DROP TABLE IF EXISTS ms_report;
CREATE TABLE ms_report  (
  report_id         bigint(20) NOT NULL DEFAULT 0                           COMMENT '上报ID',
  house_id          bigint(20) NOT NULL DEFAULT 0                           COMMENT '门牌ID',
  safety_num        int(11) NOT NULL DEFAULT 0                              COMMENT '今日居家人数',
  sure_num          int(11) NOT NULL DEFAULT 0                              COMMENT '确诊人数',
  suspected_num     int(11) NOT NULL DEFAULT 0                              COMMENT '异常疑似人数',
  normal_num        int(11) NOT NULL DEFAULT 0                              COMMENT '正常人数',
  single_num        int(11) NOT NULL DEFAULT 0                              COMMENT '隔离人数',
  remarks           varchar(200)  NOT NULL DEFAULT ''                       COMMENT '异常备注',
  report_date       date                                                    COMMENT '上报日期',
  report_status     int(11) NOT NULL DEFAULT 0                              COMMENT '上报状态:0草稿,1正常',
  ms_status         int(11) NOT NULL DEFAULT 1                              COMMENT '健康状态:1正常,2异常',
  user_create       bigint(20) NOT NULL DEFAULT 0                           COMMENT '新增人',
  time_create       datetime(0) NOT NULL DEFAULT CURRENT_TIMESTAMP(0)       COMMENT '新增时间',
  user_update       bigint(20) NOT NULL DEFAULT 0                           COMMENT '修改人',
  time_update       datetime(0) NOT NULL DEFAULT CURRENT_TIMESTAMP(0) ON UPDATE CURRENT_TIMESTAMP(0) COMMENT '修改时间',
  PRIMARY KEY (report_id) USING BTREE
) ENGINE = InnoDB default charset=utf8 comment = '每日上报表'


-- ----------------------------
-- Table structure for ms_suspected
-- ----------------------------
DROP TABLE IF EXISTS ms_suspected;
CREATE TABLE ms_suspected  (
  suspected_id      bigint(20) NOT NULL DEFAULT 0                           COMMENT '疑似ID',
  report_id         bigint(20) NOT NULL DEFAULT 0                           COMMENT '上报ID',
  user_name         varchar(50)  NOT NULL DEFAULT ''                        COMMENT '家人姓名',
  grender            int(11)    NOT NULL DEFAULT 0                          COMMENT '性别:0未设置，1男,2女',
  age               int(11)    NOT NULL DEFAULT 0                           COMMENT '年龄',
  family_status     int(11)    NOT NULL DEFAULT 0                           COMMENT '基本状态:0正常，1心血管疾病（服用ARB）,2心血管疾病（未服用ARB）,3呼吸系统病史，4肿瘤病史，5糖尿病史，6服用过激素药物，7妊娠期，8其他',
  status_desp       varchar(100)  NOT NULL DEFAULT ''                       COMMENT '状态描述',
  medical           int(11)    NOT NULL DEFAULT 0                           COMMENT '是否确诊:0否,1是',
  temperature       decimal(14,2) NOT NULL DEFAULT 0                        COMMENT '体温',
  cough             int(11)    NOT NULL DEFAULT 0                           COMMENT '咳嗽:0无咳嗽,1偶有短暂咳嗽，2咳嗽轻度影响生活，3咳嗽严重影响生活',
  muscle            int(11)    NOT NULL DEFAULT 0                           COMMENT '肌肉酸痛:0无,1按压有，2偶尔，3持续有',
  dyspnea           int(11)    NOT NULL DEFAULT 0                           COMMENT '呼吸困难:0无,1急走或上坡气短，2气短而走路变慢，3走路数分钟后气短，4气短无法离开房间',
  fatigue           int(11)    NOT NULL DEFAULT 0                           COMMENT '乏力:0无,1可体力劳动但觉得累，2轻体力劳动后长时间不能恢复，3不能正常生活',
  diarrhea          int(11)    NOT NULL DEFAULT 0                           COMMENT '腹泻:0无,1轻度腹泻少于于3次，2中度腹泻4-6次，3重度腹泻超过6次',
  single_room       int(11)    NOT NULL DEFAULT 0                           COMMENT '单间隔离:0否,1是',
  suspected_status         int(11)    NOT NULL DEFAULT 0                           COMMENT '是否疑似:0否,1是',
  others            varchar(600)  NOT NULL DEFAULT ''                       COMMENT '其他',
  user_create       bigint(20) NOT NULL DEFAULT 0                           COMMENT '新增人',
  time_create       datetime(0) NOT NULL DEFAULT CURRENT_TIMESTAMP(0)       COMMENT '新增时间',
  user_update       bigint(20) NOT NULL DEFAULT 0                           COMMENT '修改人',
  time_update       datetime(0) NOT NULL DEFAULT CURRENT_TIMESTAMP(0) ON UPDATE CURRENT_TIMESTAMP(0) COMMENT '修改时间',
  PRIMARY KEY (suspected_id) USING BTREE
) ENGINE = InnoDB default charset=utf8 comment = '家庭疑似表';