package com.bofeng.service;


import com.baomidou.mybatisplus.toolkit.IdWorker;
import com.bofeng.dao.MsReportMapper;
import com.bofeng.dao.MsSuspectedMapper;
import com.bofeng.entity.MsReport;
import com.bofeng.entity.MsSuspected;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @Author: xielianghe
 * @Date: 2020/2/4 15:44
 */

@Service
@Transactional(readOnly = true)
public class MsSuspectedService {

    @Autowired
    private MsSuspectedMapper msSuspectedMapper;
    @Autowired
    private MsReportMapper msReportMapper;

    //查询家人
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public MsSuspected selectSuspected(long suspectedId) {
        return msSuspectedMapper.selectById(suspectedId);
    }

    //添加家人
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public Long addSuspected(Long suspectedId, String userName, Integer grender, Integer age, Integer familyStatus,
                             String statusDesp, Integer medical, BigDecimal temperature, Integer cough, Integer muscle,
                             Integer dyspnea, Integer fatigue, Integer diarrhea, Integer singleRoom, String others, Long userCreate) {
        Long reportId = 0L;
        //如果是当前日期 已经添加
        Date t = new Date();
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        String reportDate = df.format(t);
        List<MsReport> listReport = msReportMapper.selectByReportDate(reportDate, userCreate);
        Long houseId = msReportMapper.selectHouseIdByUserId(userCreate);
        //首次进入。没有今日报告
        if (listReport == null || listReport.size() == 0) {
            MsReport msReport = new MsReport();
            msReport.setReportId(IdWorker.getId());
            msReport.setHouseId(houseId);
            msReport.setSafetyNum(0);
            msReport.setSureNum(0);
            msReport.setSuspectedNum(0);
            msReport.setNormalNum(0);
            msReport.setSingleNum(0);
            msReport.setRemarks("");
            msReport.setReportDate(DateTime.now().toDate());
            msReport.setMsStatus(0);
            msReport.setUserCreate(userCreate);
            msReport.setTimeCreate(DateTime.now());
            msReport.setUserUpdate(userCreate);
            msReport.setTimeUpdate(DateTime.now());
            msReportMapper.insert(msReport);
            reportId = msReport.getReportId();
        } else {
            reportId = listReport.get(0).getReportId();
        }

        //判断是否疑似
        Integer suspectedStatus = 0;
        //体温
        if (temperature.compareTo(new BigDecimal(36)) == -1 && temperature.compareTo(new BigDecimal(37.3)) == 1)
            suspectedStatus = 1;
        //咳嗽
        if (cough > 0)
            suspectedStatus = 1;
        //肌肉酸痛
        if (muscle > 0)
            suspectedStatus = 1;
        //呼吸困难
        if (dyspnea > 0)
            suspectedStatus = 1;
        //乏力
        if (fatigue > 0)
            suspectedStatus = 1;
        //腹泻
        if (diarrhea > 0)
            suspectedStatus = 1;
        if (suspectedId == 0L) {
            //添加家人
            MsSuspected msSuspected = new MsSuspected();
            msSuspected.setSuspectedId(IdWorker.getId());
            msSuspected.setReportId(reportId);
            msSuspected.setUserName(userName);
            msSuspected.setGrender(grender);//默认为家人
            msSuspected.setAge(age);
            msSuspected.setFamilyStatus(familyStatus);
            msSuspected.setStatusDesp(statusDesp);
            msSuspected.setMedical(medical);
            msSuspected.setTemperature(temperature);
            msSuspected.setCough(cough);
            msSuspected.setMuscle(muscle);
            msSuspected.setDiarrhea(dyspnea);
            msSuspected.setFatigue(fatigue);
            msSuspected.setDiarrhea(diarrhea);
            msSuspected.setSingleRoom(singleRoom);
            msSuspected.setOthers(others);
            msSuspected.setSuspectedStatus(suspectedStatus);
            msSuspected.setUserCreate(userCreate);
            msSuspected.setTimeCreate(DateTime.now());
            msSuspected.setUserUpdate(userCreate);
            msSuspected.setTimeUpdate(DateTime.now());
            msSuspectedMapper.insert(msSuspected);
        } else {
            MsSuspected msSuspected1 = msSuspectedMapper.selectById(suspectedId);
            if (msSuspected1 != null) {
                msSuspected1.setUserName(userName);
                msSuspected1.setGrender(grender);//默认为家人
                msSuspected1.setAge(age);
                msSuspected1.setFamilyStatus(familyStatus);
                msSuspected1.setStatusDesp(statusDesp);
                msSuspected1.setMedical(medical);
                msSuspected1.setTemperature(temperature);
                msSuspected1.setCough(cough);
                msSuspected1.setMuscle(muscle);
                msSuspected1.setDiarrhea(dyspnea);
                msSuspected1.setFatigue(fatigue);
                msSuspected1.setDiarrhea(diarrhea);
                msSuspected1.setSingleRoom(singleRoom);
                msSuspected1.setOthers(others);
                msSuspected1.setSuspectedStatus(suspectedStatus);
                msSuspectedMapper.updateById(msSuspected1);
            }
        }

        //更新今日日报
        updateReport(reportId, medical, singleRoom, temperature);
        return reportId;

    }

    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public Integer deleteSuspected(Long suspectedId) {

        MsSuspected msSuspected = msSuspectedMapper.selectById(suspectedId);
        if (msSuspected != null) {
            msSuspectedMapper.deleteById(suspectedId);
            //更新今日日报
            updateReport(msSuspected.getReportId(), msSuspected.getMedical(), msSuspected.getSingleRoom(), msSuspected.getTemperature());
            MsReport msReport = msReportMapper.selectById(msSuspected.getReportId());
            if (msReport != null)
                return msReport.getSafetyNum();
        }
        return 0;
    }

    //更新今日日报
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public void updateReport(Long reportId, Integer medical, Integer singleRoom, BigDecimal temperature) {
        //更新今日日报
        //确诊
        Integer isTrue = 0, isSingle = 0, isSuspected = 0, isNoSuspected = 0, isFamliy = 0;
        if (medical == 1)
            isTrue = msReportMapper.selectSuspectedNum(reportId, medical);
        //居家隔离
        if (singleRoom == 1)
            isSingle = msReportMapper.selectSingleRoomNum(reportId, singleRoom);
        //正常
        isSuspected = msReportMapper.selectisSuspectedNum(reportId);

        //疑似
        isNoSuspected = msReportMapper.selectisNoSuspectedNum(reportId);

        //今日居家
        List<MsSuspected> list = msSuspectedMapper.selectByReportId(reportId);
        if (list != null && list.size() > 0)
            isFamliy = list.size();
        MsReport msReport = msReportMapper.selectById(reportId);
        if (msReport != null) {
            msReport.setSureNum(isTrue);
            msReport.setSingleNum(isSingle);
            msReport.setSuspectedNum(isNoSuspected);
            msReport.setNormalNum(isSuspected);
            msReport.setSafetyNum(isFamliy);
            if (isNoSuspected > 0)
                msReport.setMsStatus(2);
            else
                msReport.setMsStatus(1);
            msReportMapper.updateById(msReport);
        }
    }
}
